/* mydoor.c	29.06.2000	Rolf Ludwichowski medas@t-online.de */

#define DEVICE_PATH "/dev/ttyS0"	/* Cardreader Interface */
#define BAUD_RATE    B19200		/* Baudrate (termios)   */
#define TIME_OUT     10			/* Timeout [ seconds ]  */

/* Error Exit *******************************************************/

#include <stdio.h>
#include <stdlib.h>

void perror_exit ( char *string )
  { perror ( string ); exit ( 3 ); }

void error_exit ( char *string )
  { fprintf ( stderr, "%s\n", string ); exit ( 1 ); }

/* Communication ****************************************************/

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>

static int fd;

static void set_timeout ( int seconds )
  {
  struct itimerval timervalue;
  timervalue.it_value.tv_sec = seconds;
  timervalue.it_value.tv_usec = 0;
  timervalue.it_interval.tv_sec = 0;
  timervalue.it_interval.tv_usec = 0;
  if ( setitimer ( ITIMER_REAL, &timervalue, NULL ) )
    perror_exit ( "set_timeout" );
  }

static void open_serio ( char *devicepath )
  {
  struct termios terminfo;
  if ( ( fd = open ( devicepath, O_RDWR | O_NOCTTY ) ) == -1 )
    perror_exit ( "open_serio01");
  memset ( &terminfo, 0, sizeof ( terminfo ) );
  terminfo.c_iflag = IGNBRK;
  terminfo.c_cflag = BAUD_RATE | CS8 | CREAD | CLOCAL | HUPCL;
  terminfo.c_lflag = ICANON;
  terminfo.c_cc [VTIME] = 5;
  terminfo.c_cc [VMIN] = 1;
  if ( tcsetattr ( fd, TCSANOW, &terminfo ) )
    perror_exit ( "open_serio02" );
  }

static int command_echo ( const char *string, int seconds )
  {
  int i = 0, retval = 0;
  char echo;
  set_timeout ( seconds );
  while ( string [i] != '\0' && retval == 0 )
    {
    if ( write ( fd, string+i, 1 ) == -1 )
      perror_exit ( "command_echo01" );
    if ( read ( fd, &echo, 1 ) == -1 )
      perror_exit ( "command_echo02" );
    if ( string [i++] != echo ) retval = 1;
    }
  set_timeout ( 0 );
  return retval;
  }

static void read_buffer ( unsigned char *buffer, int length, int seconds )
  {
  int i;
  set_timeout ( seconds );
  for ( i = 0; i < length; i++ )
    if ( read ( fd, buffer+i, 1 ) == -1 )
      perror_exit ( "read_buffer" );
  set_timeout ( 0 );
  }

static void print_data ( unsigned char *buffer, int length, int seconds )
  {
  int i;
  set_timeout ( seconds );
  for ( i = 0; i < length; i++ ) printf ( "%X:", buffer [i] );
  putchar ( '\n' );
  set_timeout ( 0 );
  }

/* Main Program *****************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <setjmp.h>
#include <signal.h>

static sigjmp_buf env;

static void terminate ( int sig )
  {
  exit ( 0 );
  }

static void timeout ( int sig )
  {
  fprintf ( stderr, "timeout\n" );
  siglongjmp ( env, sig );
  }

#define LENGTH 9

main ( int argc, char *argv[] )
  {
  unsigned char code, buffer [ LENGTH ];

  if ( argc != 2 )
    error_exit ( "so geht das nicht" );		/* dummy Argument */

  signal ( SIGTERM, terminate );
  signal ( SIGALRM, timeout );
  open_serio ( DEVICE_PATH );

  sigsetjmp ( env, 1 );

  while (1)
    {
    read_buffer ( &code, 1, 0 );
    if ( code != '+' ) continue;
    if ( command_echo ( "les", TIME_OUT ) ) continue;
    read_buffer ( buffer, LENGTH, TIME_OUT );
    read_buffer ( &code, 1, TIME_OUT );
    if ( code == '-' ) print_data ( buffer, LENGTH, TIME_OUT );
    }
  }

/* The End **********************************************************/
